/*****************************************************************************
  *                                                                           
  *  Copyright (c) 2008 Johannes Hermen www.der-frickler.net                   
  *                                                                           
  *                                                                           
  *  This library is free software; you can redistribute it and/or modify it  
  *  under the terms of the GNU Lesser General Public License as published    
  *  by the Free Software Foundation; either version 2 of the License, or     
  *  (at your option) any later version.                                      
  *                                                                           
  *  This software is distributed in the hope that it will be useful, but     
  *  WITHOUT ANY WARRANTY; without even the implied warranty of               
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        
  *  Lesser General Public License for more details.                          
  *                                                                           
  *  You should have received a copy of the GNU Lesser General Public         
  *  License along with this library; if not, write to the Free Software      
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  
  *                                                                           
  *****************************************************************************
  *                                                                           
  *  Servo-Tester ATMega 8 
  *  Generiert ein Signal das die angeschlossenen Servos erst langsam dann 
  *  schnell von Endausschlag zu Endausschlag dreht. Eignet sich zum einlaufen 
  *  von Servos sowie (bei mir) zum finden von Servos die hin und wieder auf 
  *  einer Position hängenbleiben.
  *                                                                           
  *  PINOUT: Servo Signal OUT: PB0-PB7 Pins 14-19
  *          Control LED: PD0  Pin 2
  *                                                                           
  *****************************************************************************/


#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>

uint16_t value;
uint16_t j; 
uint8_t  state = 0;
uint8_t  isBreak = 0;


// on timer1 overflow
ISR (TIMER1_OVF_vect)
{
	setLEDs();

	if (PORTB == 255) {
		PORTB = 0;
		TCNT1 = 65535 - 18000;
    } else {
		PORTB = 255;
		TCNT1 = 65535 - value;
    }
	TCCR1B = 1;		   // start timer1
}

// on INT0 / PD2 pinchange
ISR (INT0_vect)
{
	if ((PIND & 4)==4) 	// check 3th bit of PORTD (pin 4) got HIGH
  	{
		isBreak = 1;
		if (state == 6) {
			state = 0;	
		} else {
			state ++;
		}
  	}
}

void setPos(uint16_t p_value) {
	j = 200;
	value = p_value;
	while ( (0<j--) && !isBreak) {
		waitMS();
	}
}

void setLEDs() {
	if (value >= 1450 && value <= 1550) // set middle LED on
		PORTD = 224;
	else   				// set middle LED off
		PORTD = 0;
}

void waitMS() {
	_delay_ms(1);
}

int main (void)
{
  	DDRB = 255;	// portb as output
	PORTB = 0; 	// portb to 0	

  	DDRD = 251;	// portd as output, int0 as input 
	PORTD = 0; 	// portd to 0	

    DDRC = 0;    // portc as input
    PORTC = 255; // portc to 0

    MCUCR = 1;	// External Interrupt on PinChange
    GICR = 64;	// enable Int0

    TCNT1 = 0;
    TIMSK = 4;	// 16-Bit Timer Overflow-Interrupt an
    TCCR1A = 0;	// 16-Bit Normal Mode
    TCCR1B = 0;	// stop 16-Bit Timer 

  	sei(); 	// Interrupts on.

	// set value to center pos
	value = 1500;

	// init timer1
	TCNT1 = 65535 - value;
	TCCR1B = 1;		   // start timer1
	PORTB = 255;


	while(1)
	{
		isBreak = 0;
		if (state == 0) {
			value = 1500;
		} else if (state == 1) {
			value = 1000;
		} else if (state == 2) {
			value = 2000;
		} else if (state == 3) {
			value = 1500;
		} else if (state == 4 || state == 6) {

			// turn from left to right
			while ((value < 2000) && !isBreak) {
				value = value + 1;
				waitMS();
			}

			// turn from right to left
			while ((value > 1000) && !isBreak) {
				value = value - 1;
				waitMS();
			}

		} if (state > 4) {

			setPos(1000);
			setPos(1500);
			setPos(2000);
			setPos(1500);
			setPos(1000);

		}

	}
}
